﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System;
using System.IO;
using System.Threading;
using System.Windows;
using PhotoApp.Utility;

namespace Logger
{
    /// <remarks>
    /// Defines logger which allows to log different messages with parameters.
    /// </remarks>
    class AppLogger
    {
        //Lock
        private static ReaderWriterLock readerWriterLock = new ReaderWriterLock();

        /// <summary>
        /// Writes the log to file
        /// </summary>
        /// <param name="logMessage">Message to write</param>
        public static void LogToFile(string logMessage)
        {
            try
            {
                readerWriterLock.AcquireWriterLock(Timeout.Infinite);
                using (StreamWriter w = File.AppendText(AppConstants.AppLogFileName))
                {
                    w.Write("\r\nLog Entry : ");
                    w.WriteLine("{0} {1}", DateTime.Now.ToLongTimeString(),
                        DateTime.Now.ToLongDateString());

                    w.WriteLine("  {0}", logMessage);
                    w.WriteLine("-------------------------------");

                    // Update the underlying file.
                    w.Flush();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }
            finally
            {
                readerWriterLock.ReleaseWriterLock();
            }
        }
    }
}
