﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System;
using LocationApiLib;

namespace PhotoApp.Utility
{

    /// <summary>
    /// This class provides methods to start or stop geo location events.
    /// Class is derived from 'ILocationEvents' interface which has methods to get notification while position or status changed.
    /// We have used COM "LocationApi Type Library" as a reference.
    /// </summary>
    class GeoLocationHelper : ILocationEvents
    {
        //Location class's object to get geo position.
        Location geoLoc;
        //Guid for Lat-Long report.
        Guid guidLatLong = new Guid("7FED806D-0EF8-4F07-80AC-36A0BEAE3134");
        //Guid for civic address report.
        //Guid guidCivicAddress = new Guid("C0B19F70-4ADF-445d-87F2-CAD8FD711792");
        //Event handler to get the geo postion changed event.
        public event EventHandler<GeoPositionChangedEventArgs> positionChanged;
        //Event handler to get the GPS device status changed event.
        public event EventHandler statusChanged;

        bool registeredForReport;

        /// <summary>
        /// Starts to get location update.
        /// </summary>
        public void StartGettingLocationUpdate()
        {
            //Start to get geo position.
            if (geoLoc == null)
            {
                geoLoc = new Location();
                
                
                /*if (geoLoc != null)
                {
                    geoLoc.RequestPermissions(default(_RemotableHandle), ref guidLatLong, 1, 0);
                }*/
            }

            if (geoLoc != null && !registeredForReport)
            {
                geoLoc.RegisterForReport(this, guidLatLong, 0);
                registeredForReport = true;
            }
        }

        /// <summary>
        /// Stops to get location update.
        /// </summary>
        public void StopGettingLocationUpdate()
        {
            if (geoLoc != null && registeredForReport)
            {
                geoLoc.UnregisterForReport(guidLatLong);
                registeredForReport = false;
            }
        }

        /// <summary>
        /// Event invoked when geo position changed.
        /// </summary>
        /// <param name="reportType">Guid of report type</param>
        /// <param name="pLocationReport">Location report</param>
        public void OnLocationChanged(ref Guid reportType, ILocationReport pLocationReport)
        {
            if (reportType == guidLatLong)
            {
                ILatLongReport lr = (ILatLongReport)pLocationReport;

                if (lr != null && positionChanged != null)
                    positionChanged(this, new GeoPositionChangedEventArgs(lr));
            }
        }

        /// <summary>
        /// Event invoked when status for GPS changed.
        /// </summary>
        /// <param name="reportType">Guid of report type</param>
        /// <param name="newStatus">Status of report</param>
        public void OnStatusChanged(ref Guid reportType, LOCATION_REPORT_STATUS newStatus)
        {
            if (statusChanged != null)
                statusChanged(this, new EventArgs());
        }
    }
}
