/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.

The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.

Copyright (c) 2008, Meraka Institute

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

 * Neither the name of the Meraka Institute nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intels suppliers or licensors in any way.

*/

#include "stdafx.h"
#include "RotateVideo.h"

RotateVideo::RotateVideo() : CTransformFilter(NAME("PhotoApp Rotate Video Filter"),0,CLSID_RotateVideoFilter),
											m_pRotate(NULL),
											m_nBytesPerPixel(3),
											m_nInHeight(0),
											m_nInWidth(0),
											m_nInPixels(0),
											m_nOutHeight(0),
											m_nOutWidth(0),
											m_nOutPixels(0),
											m_nBitCount(0)
{
	DbgLog((LOG_TRACE,3,TEXT("RotateVideo::RotateVideo() called")));
	//Call the initialise input method to load all acceptable input types for this filter
	//InitialiseInputTypes();
	m_pRotate = new PhotoAppRotateLogic();
	m_nRotation = 0;
	m_nOutWidth = 0;
	m_nOutHeight = 0;
	m_nStride = 0;
	m_nPadding = 0;
}

RotateVideo::~RotateVideo()
{
	
}

CUnknown* WINAPI RotateVideo::CreateInstance( LPUNKNOWN pUnk, HRESULT *pHr )
{
	RotateVideo *pFilter = new RotateVideo();
	if (pFilter== NULL) 
	{
		*pHr = E_OUTOFMEMORY;
	}
	return pFilter;
}

STDMETHODIMP RotateVideo::NonDelegatingQueryInterface( REFIID riid, void **ppv )
{
	if(riid == (IID_IRotateVideoInterface))
	{
		//return GetInterface((IRotateVideoInterface*) this, ppv);
		return GetInterface(static_cast<IRotateVideoInterface*>(this),ppv);
	}
	else
	{
		return CTransformFilter::NonDelegatingQueryInterface(riid, ppv);
	}
}

STDMETHODIMP RotateVideo::SetRotationAngle(int rAngle)
{
		m_nRotation = rAngle;
		if(m_pRotate != NULL)
			m_pRotate->SetRotateMode((ROTATE_MODE)m_nRotation);
		return S_OK;
}

HRESULT RotateVideo::CheckInputType(const CMediaType *mtIn)
{
	return S_OK;
}

HRESULT RotateVideo::SetMediaType( PIN_DIRECTION direction, const CMediaType *pmt )
{
	if (direction == PINDIR_INPUT)
	{
		if (pmt->formattype == FORMAT_VideoInfo)
		{
			VIDEOINFOHEADER* pVih = (VIDEOINFOHEADER*) pmt->pbFormat;
			CopyMemory(&m_videoInHeader, pVih, sizeof(VIDEOINFOHEADER));
			//Set these values now so we don't have to repeat it for each transform
			m_nInHeight = abs(m_videoInHeader.bmiHeader.biHeight);
			m_nInWidth = m_videoInHeader.bmiHeader.biWidth;
			m_nInPixels = m_nInHeight * m_nInWidth;
			m_nBitCount = m_videoInHeader.bmiHeader.biBitCount;
			if(m_nBitCount != 24)
				return VFW_E_NO_ACCEPTABLE_TYPES;
			if (pmt->subtype == MEDIASUBTYPE_RGB24)
			{
				//m_pRotate = new PicRotateRGB24Impl();
				m_nBytesPerPixel = 3; //BYTES_PER_PIXEL_RGB24;
			}
			else if (pmt->subtype == MEDIASUBTYPE_RGB32)
			{
				//m_pRotate = new PicRotateRGB32Impl();
				m_nBytesPerPixel = 4; //BYTES_PER_PIXEL_RGB32;
			}
			else if(pmt->subtype == MEDIASUBTYPE_YUY2)
			{
				m_nBytesPerPixel = 2;
			}
			
		}
	}
	else   // output pin
	{
		ASSERT(direction == PINDIR_OUTPUT);
		if (pmt->formattype == FORMAT_VideoInfo)
		{
			// Store a copy of the video header info
			VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)pmt->pbFormat;
			CopyMemory(&m_videoOutHeader, pVih, sizeof(VIDEOINFOHEADER));
			//Set these values now so we don't have to repeat it for each transform
			m_nOutHeight = abs(m_videoOutHeader.bmiHeader.biHeight);
			m_nOutWidth = m_videoOutHeader.bmiHeader.biWidth;
			m_nOutPixels = m_nOutHeight * m_nOutWidth;
		}
	}
	return S_OK;
}

HRESULT RotateVideo::GetMediaType( int iPosition, CMediaType *pMediaType )
{
	DbgLog((LOG_TRACE,3,TEXT("RotateVideo::GetMediaType called")));
	if(!m_pInput->IsConnected())
	{
		return E_INVALIDARG;
	}

	if(iPosition < 0)
		return E_INVALIDARG;
	else if(iPosition == 0)
	{
		HRESULT hr = m_pInput->ConnectionMediaType(pMediaType);
		if (FAILED(hr))
		{
			return hr;
		}
		
		ASSERT(pMediaType->formattype == FORMAT_VideoInfo);
		if(pMediaType->formattype != FORMAT_VideoInfo)
		{
			return E_INVALIDARG;
		}
		VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)pMediaType->pbFormat;

		BITMAPINFOHEADER *pBi = &(pVih->bmiHeader);
		//m_nRotation = 6 ; //1,,6
		switch (m_nRotation)
		{
			case ROTATE_NONE:
			case ROTATE_180_DEGREES_CLOCKWISE:
			case ROTATE_FLIP_HORIZONTAL:
			case ROTATE_FLIP_VERTICAL:
				{
					m_nOutWidth = m_nInWidth;
					m_nOutHeight = m_nInHeight;
					break;
				}
			case ROTATE_90_DEGREES_CLOCKWISE:
			case ROTATE_270_DEGREES_CLOCKWISE:
			case ROTATE_FLIP_DIAGONALLY:
				{
					m_nOutWidth = m_nInHeight;
					m_nOutHeight = m_nInWidth;
					break;
				}
		}
			pBi->biHeight = m_nOutHeight;
			ASSERT(pBi->biHeight > 0);

			pBi->biWidth = m_nOutWidth;
			ASSERT(pBi->biWidth > 0);
			pBi->biSizeImage = pBi->biWidth * pBi->biHeight * m_nBytesPerPixel;

			pVih->rcSource.top = 0;
			pVih->rcSource.left = 0;
			pVih->rcSource.right = pBi->biWidth;
			pVih->rcSource.bottom = pBi->biHeight;

			pVih->rcTarget.top = 0;
			pVih->rcTarget.left = 0;
			pVih->rcTarget.right = pBi->biWidth;
			pVih->rcTarget.bottom = pBi->biHeight;

			pMediaType->lSampleSize = pBi->biSizeImage;
	
			RecalculateFilterParameters();
			return S_OK;		
	}
	return VFW_S_NO_MORE_ITEMS;
}

void RotateVideo::RecalculateFilterParameters()
{
	if (m_nBytesPerPixel == 3)
	{
		m_nStride =  (m_nOutWidth * (m_nBitCount / 8) + 3) & ~3;
		m_nPadding = m_nStride - (m_nBytesPerPixel * m_nOutWidth);
	}
	else if (m_nBytesPerPixel == 2)
	{
		m_nStride =  (m_nOutWidth * (m_nBitCount / 8) + 2) & ~2;
		m_nPadding = m_nStride - (m_nBytesPerPixel * m_nOutWidth);
	}
	else if(m_nBytesPerPixel == 4)
	{
		m_nStride =  (m_nOutWidth * (m_nBitCount >> 3));
		m_nPadding = 32 - (m_nStride%32);
		//m_nStride += m_nStride % 32;
	}
}

HRESULT RotateVideo::DecideBufferSize( IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProp )
{
	// Adding padding to take stride into account
	pProp->cbBuffer = (m_nOutWidth + m_nPadding) * m_nOutHeight * m_nBytesPerPixel;

	if (pProp->cbAlign == 0)
	{
		pProp->cbAlign = 1;
	}
	if (pProp->cBuffers == 0)
	{
		pProp->cBuffers = 1;
	}
	
	// Set allocator properties.
	ALLOCATOR_PROPERTIES Actual;
	HRESULT hr = pAlloc->SetProperties(pProp, &Actual);
	if (FAILED(hr)) 
	{
		return hr;
	}
	// Even when it succeeds, check the actual result.
	if (pProp->cbBuffer > Actual.cbBuffer) 
	{
		return E_FAIL;
	}
	return S_OK;
}

HRESULT RotateVideo::CheckTransform( const CMediaType *mtIn, const CMediaType *mtOut )
{
		//Make sure the input and output types are related
	if (mtOut->majortype != MEDIATYPE_Video)
	{
		return VFW_E_TYPE_NOT_ACCEPTED;
	}
	if (mtIn->subtype == MEDIASUBTYPE_RGB24)
		if (mtOut->subtype != MEDIASUBTYPE_RGB24)
			return VFW_E_TYPE_NOT_ACCEPTED;
		else
		{
		  if (mtOut->formattype == FORMAT_VideoInfo)
		  {
			VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)mtOut->pbFormat;
			//Now we need to calculate the size of the output image
			BITMAPINFOHEADER* pBi = &(pVih->bmiHeader);
			int iNewStride = pBi->biWidth;
			// TEMP HACK TO TEST IF IMAGE RENDERS CORRECTLY
			if (iNewStride != m_nOutWidth)
			{
			  //m_nStrideRenderer = iNewStride;
			  // FOR NOW JUST REJECT THE TYPE AND LET THE COLOR CONVERTER HANDLE IT
			  return VFW_E_TYPE_NOT_ACCEPTED;
			}
			int iNewWidth = pVih->rcSource.right - pVih->rcSource.left;
			int iNewHeight = pBi->biHeight;
			if (iNewHeight < 0)
			{
			  // REJECT INVERTED PICTURES
			  return VFW_E_TYPE_NOT_ACCEPTED;
			}
		  }
		}

	if (mtOut->subtype == MEDIASUBTYPE_RGB32)
	{
		VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)mtOut->pbFormat;
		//Now we need to calculate the size of the output image
		BITMAPINFOHEADER* pBi = &(pVih->bmiHeader);
		int iNewStride = pBi->biWidth;
		// TEMP HACK TO TEST IF IMAGE RENDERS CORRECTLY
		if (iNewStride != m_nOutWidth)
		{
			//m_nStrideRenderer = iNewStride;
			// FOR NOW JUST REJECT THE TYPE AND LET THE COLOR CONVERTER HANDLE IT
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
		int iNewWidth = pVih->rcSource.right - pVih->rcSource.left;
		int iNewHeight = pBi->biHeight;
		if (iNewHeight < 0)
		{
			// REJECT INVERTED PICTURES
			return VFW_E_TYPE_NOT_ACCEPTED;
		}
	}

// 	if (mtIn->subtype == MEDIASUBTYPE_RGB32)
// 		if (mtOut->subtype != MEDIASUBTYPE_RGB32)
// 			return VFW_E_TYPE_NOT_ACCEPTED;

	if (mtOut->formattype != FORMAT_VideoInfo)
	{
		return VFW_E_TYPE_NOT_ACCEPTED;
	}
	return S_OK;
}

HRESULT RotateVideo::Transform( IMediaSample *pSource, IMediaSample *pDest )
{
	// Get pointers to the underlying buffers.
	long lSrcDataLen = pSource->GetSize();
	long lDestDataLen = pDest->GetSize();

	BYTE *pBufferIn, *pBufferOut;
	HRESULT hr = pSource->GetPointer(&pBufferIn);
	if (FAILED(hr))
	{
		return hr;
	}
	
	hr = pDest->GetPointer(&pBufferOut);
	if (FAILED(hr))
	{
		return hr;
	}
	// Process the data.

	DWORD cbDest = ApplyTransform(pBufferIn, pBufferOut);
	
	KASSERT((long)cbDest <= pDest->GetSize());

	pDest->SetActualDataLength(cbDest);
	pDest->SetSyncPoint(TRUE);
	return S_OK;
}

DWORD RotateVideo::ApplyTransform( BYTE* pBufferIn, BYTE* pBufferOut )
{
	int nTotalSize = 0;
	//make sure we were able to initialise our converter
	if (m_pRotate)
	{
		// Create temp buffer for crop
		// TODO: Add stride parameter for rotate class to avoid this extra mem alloc
		BYTE* pBuffer = new BYTE[m_nOutWidth * m_nOutHeight * m_nBytesPerPixel];

		m_pRotate->SetInDimensions(m_nInWidth, m_nInHeight);
		m_pRotate->SetRotateMode((ROTATE_MODE)m_nRotation);
		m_pRotate->Rotate((void*)pBufferIn, (void*)pBuffer,m_nBytesPerPixel);

		// Copy the cropped image with stride padding
		BYTE* pFrom = pBuffer;
		BYTE* pTo = pBufferOut;

		int nBytesPerLine = m_nOutWidth * m_nBytesPerPixel;
		for (size_t i = 0; i < m_nOutHeight; i++)
		{
			memcpy(pTo, pFrom, nBytesPerLine);
			pFrom += nBytesPerLine;
			pTo += nBytesPerLine;

			//if (m_nBytesPerPixel == BYTES_PER_PIXEL_RGB24)
			if (m_nBytesPerPixel == 3)
			{
				for (size_t j = 0; j < m_nPadding; j++)
				{
					//*pTo = 0;
					//pTo++;
				}
			}
			//else if (m_nBytesPerPixel == BYTES_PER_PIXEL_RGB32)
			else if (m_nBytesPerPixel == 4)
			{
				// TESTING SO FAR HAS SHOWN THAT NO PADDING IS NECCESSARY FOR RGB32???
				//int nPadding = 0;//(m_nStrideRenderer - m_nOutWidth)*4;
 			//	for (size_t j = 0; j < nPadding; j++)
 			//	{
 			//		*pTo = 0;
 			//		pTo++;
 			//	}
			}
		}
		nTotalSize = (m_nOutWidth + m_nPadding) * m_nOutHeight * m_nBytesPerPixel;
		delete[] pBuffer;
	}
	else
	{
		DbgLog((LOG_TRACE, 0, TEXT("Rotator is not initialised - unable to rotate")));
	}
	return nTotalSize;
}

void RotateVideo::SetInDimensions(int nWidth, int nHeight)
	{
		m_nWidth = nWidth;
		m_nHeight = nHeight;
	}

void RotateVideo::SetRotateMode(ROTATE_MODE eMode)
	{
		m_eMode = eMode;
	}

	/// Implementation must be overidden by sub class
	//virtual bool Rotate(void* pInImg, void* pOutImg) = 0;
bool RotateVideo::Rotate( void* pInImg, void* pOutImg )
{
	if (!pInImg || !pOutImg) return false;

	//int nBytesPerPixel = BytesPerPixel();
	int nBytesPerPixel = 3;
	switch (m_eMode)
	{
			case ROTATE_NONE:
				{
					memcpy(pOutImg, pInImg, m_nWidth * m_nHeight * nBytesPerPixel);
					return true;
				}
			case ROTATE_90_DEGREES_CLOCKWISE:
				{
					BYTE* pSrc = (BYTE*)pInImg;
					BYTE* pDest = NULL;
					BYTE* pSrcCol = NULL;
					for (int i = 0; i < m_nWidth; ++i, pSrc += nBytesPerPixel)
					{
						int nNewRow = (m_nWidth - i);//Index Base 1
						int nNewIndex = ((nNewRow - 1) * m_nHeight);// Base 1
						pSrcCol = pSrc;
						pDest = (BYTE*)pOutImg + (nNewIndex*nBytesPerPixel);
						for (int j = 0; j < m_nHeight; ++j, pSrcCol += (nBytesPerPixel*m_nWidth), pDest += nBytesPerPixel)
						{
							memcpy(pDest, pSrcCol, nBytesPerPixel);
						}
						// Copy pixels in same column to their destinations
					}
					return true;
				}
			case ROTATE_180_DEGREES_CLOCKWISE:
				{
					BYTE* pSrc = (BYTE*)pInImg;
					int nTotalPixels = m_nWidth * m_nHeight; 
					int nLastIndex = (nTotalPixels - 1) * nBytesPerPixel;
					BYTE* pDest = (BYTE*)pOutImg + nLastIndex;
					for (int i = 0; i < nTotalPixels; ++i, pSrc += nBytesPerPixel, pDest -= nBytesPerPixel)
					{
						memcpy(pDest, pSrc, sizeof(BYTE) *nBytesPerPixel);
					}
					return true;
				}

			case ROTATE_270_DEGREES_CLOCKWISE:
				{
					int iTargetRowLength = m_nHeight * nBytesPerPixel;
					BYTE* pSrc = (BYTE*)pInImg;
					BYTE* pDest = (BYTE*)pOutImg + iTargetRowLength - nBytesPerPixel;

					for ( int y = 0; y < m_nHeight; y++ )
					{
						BYTE* pDestPixel = pDest;
						for (int x = 0; x < m_nWidth; ++x, pSrc += nBytesPerPixel, pDestPixel += iTargetRowLength)
						{
							memcpy(pDestPixel, pSrc, nBytesPerPixel);
						}
						pDest -= nBytesPerPixel;
					}
					return true;
				}
			case ROTATE_FLIP_VERTICAL:
				{
					// Code to flip image
					int nRowLength = m_nWidth * nBytesPerPixel;
					BYTE* pSrc = (BYTE*)pInImg;
					//BYTE* pDest = (BYTE*)pOutImg + ((m_nHeight * nRowLength ) - nRowLength);
					BYTE* pDest = (BYTE*)pOutImg + ((m_nHeight - 1) * nRowLength);
					for (int i = 0; i < m_nHeight; ++i, pSrc += nRowLength, pDest -= nRowLength)
					{
						memcpy(pDest, pSrc, nRowLength);
					}
					return true;
				}
			case ROTATE_FLIP_HORIZONTAL:
				{
					int iRowLength = m_nWidth * nBytesPerPixel;
					BYTE* pSrc = (BYTE*)pInImg;
					BYTE* pDest = (BYTE*)pOutImg + iRowLength - nBytesPerPixel;

					for ( int y = 0; y < m_nHeight; y++ )
					{
						BYTE* pDestPixel = pDest;
						for ( int x = 0; x < m_nWidth; x++, pSrc += nBytesPerPixel, pDestPixel-=nBytesPerPixel )
						{
							memcpy(pDestPixel, pSrc, nBytesPerPixel);
						}
						pDest += iRowLength;
					}
					return true;
				}
			case ROTATE_FLIP_DIAGONALLY:
				{
					BYTE* pSrc = (BYTE*)pInImg;
					BYTE* pDest = NULL;
					BYTE* pSrcCol = NULL;
					for (int i = 0; i < m_nWidth; ++i, pSrc += nBytesPerPixel)
					{
						int nNewRow = (m_nWidth - i);//Index Base 1
						// Select target row to be one higher and subtract sizeof RGB block to be at last index of row
						++nNewRow;
						int nNewIndex = ((nNewRow - 1) * m_nHeight);// Base 1
						// Adjust index to previous row
						--nNewIndex;
						pSrcCol = pSrc;
						pDest = (BYTE*)pOutImg + (nNewIndex*nBytesPerPixel);
						for (int j = 0; j < m_nHeight; ++j, pSrcCol += (nBytesPerPixel*m_nWidth), pDest -= nBytesPerPixel)
						{
							memcpy(pDest, pSrcCol, nBytesPerPixel);
						}
						// Copy pixels in same column to their destinations
					}
					return true;
				}
			default:
				{
					// Unimplemented
					return false;
				}
		}
}
