/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.

The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.

Copyright (c) 2008, Meraka Institute

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

 * Neither the name of the Meraka Institute nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intels suppliers or licensors in any way.

*/

#include <streams.h>
#include <windows.h>
#include <initguid.h>
#include "PhotoAppRotateLogic.h"
#include "RotateVideoSettingsInterface.h"

#pragma once
class RotateVideo : public CTransformFilter, public IRotateVideoInterface
{
public:
	RotateVideo(void);
	~RotateVideo(void);
	
	static CUnknown* WINAPI CreateInstance(LPUNKNOWN pUnk, HRESULT *pHr);

	HRESULT RotateVideo::CheckInputType(const CMediaType *mtIn);
	HRESULT RotateVideo::GetMediaType( int iPosition, CMediaType *pMediaType );
	HRESULT RotateVideo::SetMediaType( PIN_DIRECTION direction, const CMediaType *pmt );
	void RotateVideo::RecalculateFilterParameters();
	HRESULT RotateVideo::DecideBufferSize( IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pProp );
	HRESULT RotateVideo::CheckTransform( const CMediaType *mtIn, const CMediaType *mtOut );
	DWORD RotateVideo::ApplyTransform( BYTE* pBufferIn, BYTE* pBufferOut );
	HRESULT RotateVideo::Transform( IMediaSample *pSource, IMediaSample *pDest );
	bool RotateVideo::Rotate( void* pInImg, void* pOutImg );
	void RotateVideo::SetRotateMode(ROTATE_MODE eMode);
	void RotateVideo::SetInDimensions(int nWidth, int nHeight);
	STDMETHODIMP RotateVideo::SetRotationAngle(int rAngle);
public:
	DECLARE_IUNKNOWN
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv);
	
	int m_nBytesPerPixel;
	/// Pointer to our picture rotation class
	PhotoAppRotateLogic* m_pRotate;
	/// 0 = Don't rotate
	/// 1 = 90 deg Clockwise
	/// 2 = 180 deg Clockwise
	/// 3 = 270 deg Clockwise
	/// 4 = Upside down
	/// 5 = Mirror
	/// 6 = Flip diagonally
	int m_nRotation; 

	int m_nStride;
	int m_nPadding;

	/// Width of input image
	int m_nWidth;
	/// Height of input image
	int m_nHeight;
	/// Rotation mode
	ROTATE_MODE m_eMode;
	
	/// Video header of input
	VIDEOINFOHEADER m_videoInHeader;
	/// Video header of output
	VIDEOINFOHEADER m_videoOutHeader;
	// Other video information
	/// Height of image
	int m_nInHeight;
	/// Width of image
	int m_nInWidth;
	/// Number of pixels in image
	int m_nInPixels;
	/// Height in output image
	int m_nOutHeight;
	/// Width in output image
	int m_nOutWidth;
	/// Number of pixels in output image
	int m_nOutPixels;
	/// Bit count of image
	int m_nBitCount;
};