/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.

The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.

Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intels suppliers or licensors in any way.

*/

#include "stdafx.h"
#include "PreviewCapture.h"
#include "RotateVideoSettingsInterface.h"

PreviewCapture* PreviewCapture::pPreviewCapture = NULL;

PreviewCapture::PreviewCapture(void)
{
}


PreviewCapture::~PreviewCapture(void)
{
}

PreviewCapture* PreviewCapture::SharedInstance()
{
	
	if(pPreviewCapture == NULL)
	{
		pPreviewCapture = new PreviewCapture();
	}
	
	return pPreviewCapture;
}

HRESULT PreviewCapture::InitCaptureGraphBuilder()
{
	IGraphBuilder *pLocalGraph = NULL;
	ICaptureGraphBuilder2 *pLocalBuild = NULL;
	//Create capture graph builder
	HRESULT hr = CoCreateInstance(CLSID_CaptureGraphBuilder2,NULL,CLSCTX_INPROC_SERVER,IID_ICaptureGraphBuilder2,(void**)&pLocalBuild);
	if(SUCCEEDED(hr))
	{
		hr = CoCreateInstance(CLSID_FilterGraph,NULL,CLSCTX_INPROC_SERVER,IID_IGraphBuilder,(void**)&pLocalGraph);
		if(SUCCEEDED(hr))
		{
			pLocalBuild->SetFiltergraph(pLocalGraph);
			this->pBuild = pLocalBuild;
			this->pGraph = pLocalGraph;
			return S_OK;
		}
		else
		{
			pLocalBuild->Release();
		}
	}
	return hr; //FAILED
}

HRESULT PreviewCapture::EnumerateDevices()
{
	REFGUID category = CLSID_VideoInputDeviceCategory;
	IEnumMoniker *ppLocalEnum = NULL;
	ICreateDevEnum *pLocalDevEnum = NULL;

    // Create the System Device Enumerator.
    HRESULT hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL,  
        CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&pLocalDevEnum));

    if (SUCCEEDED(hr))
    {
        // Create an enumerator for the category.
        hr = pLocalDevEnum->CreateClassEnumerator(category,&ppLocalEnum, 0);
        if (hr == S_FALSE)
        {
            hr = VFW_E_NOT_FOUND;  // The category is empty. Treat as an error.
		
		}
		else
		{
			this->pEnum = ppLocalEnum;
		}
        pLocalDevEnum->Release();
    }
    return hr;
}
HRESULT PreviewCapture::AddSampleGrabber()
{
	ISampleGrabber *pGrabber = NULL;
	IBaseFilter *pGrabberBase = NULL;
	
	HRESULT hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER,
		IID_PPV_ARGS(&pGrabberBase));

	//HRESULT hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER,IID_ISampleGrabber,(void**)&pGrabberBase);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pGrabberBase = pGrabberBase;
	/*
	hr = pGraph->QueryInterface(IID_PPV_ARGS(&pEvent));
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	hr = pGraph->AddFilter(pGrabberBase, L"Sample Grabber");
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	//hr = pGrabberF->QueryInterface(IID_PPV_ARGS(&pGrabber));
	hr = pGrabberBase->QueryInterface(IID_ISampleGrabber,(void**)&pGrabber);
	this->pGrabber = pGrabber;
	if(FAILED(hr))
	{
		return E_FAIL;
	}

	
	AM_MEDIA_TYPE mt;
	ZeroMemory(&mt,sizeof(mt));
	mt.majortype = MEDIATYPE_Video;
	mt.subtype = MEDIASUBTYPE_RGB24;
	

	//Find supported resolutions and set th highest available
	int iCount, iSize;
	IAMStreamConfig *pStrmConfig = NULL;
	long maxSupportedHeight = 0;
	long maxSUpportedWidth = 0;

	hr = this->pBuild->FindInterface(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,this->pCap,IID_IAMStreamConfig, (void**)&pStrmConfig);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	hr = pStrmConfig->GetNumberOfCapabilities(&iCount,&iSize);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	
	if(iSize == sizeof(VIDEO_STREAM_CONFIG_CAPS))
	{
	
		for(int i=0; i < iCount;i++)
		{
			VIDEO_STREAM_CONFIG_CAPS scc;
			AM_MEDIA_TYPE *pmt = NULL;
			VIDEOINFOHEADER *pVih = NULL;

			hr = pStrmConfig->GetStreamCaps(i,&pmt,(BYTE*)&scc);
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			if(pmt->majortype == MEDIATYPE_Video)
			{
				pVih = (VIDEOINFOHEADER*) pmt->pbFormat;
				long h = pVih->bmiHeader.biHeight;
				long w = pVih->bmiHeader.biWidth;
				if(w > maxSUpportedWidth)
				{
					maxSUpportedWidth = w;
					maxSupportedHeight = h;
					//now set highest supported resolutio
					hr = pStrmConfig->SetFormat(pmt);
					if(FAILED(hr))
					{
						return E_FAIL;
					}
					if(FAILED(hr))
					{
						return E_FAIL;
					}
					
					hr = pGrabber->SetMediaType(pmt);
					if(FAILED(hr))
					{
						return E_FAIL;
					}

					if(h == 720)
					{
						break;
					}
				}
			}//if
		}//for
	}//if

	hr = pGrabber->SetMediaType(&mt);
	if(FAILED(hr))
	{
		return E_FAIL;
	}

	hr = pGrabber->SetOneShot(FALSE);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	hr = pGrabber->SetBufferSamples(TRUE);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	return hr;
}

HRESULT PreviewCapture::GrabBitmap(LPCWSTR pFileName)
{		
	if(this->_mediaControl == NULL || this->pEvent == NULL || this->pGrabber == NULL)
		return E_FAIL;


	AM_MEDIA_TYPE mt;
	ZeroMemory(&mt,sizeof(mt));
	mt.majortype = MEDIATYPE_Video;
	//mt.subtype = MEDIASUBTYPE_RGB24;

	/*
	HRESULT hr = this->pGrabber->SetMediaType(&mt);

	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	HRESULT hr = this->pGrabber->GetConnectedMediaType(&mt);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	/*
	long evCode;
	hr = this->pEvent->WaitForCompletion(INFINITE, &evCode);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	long cbBuffer;
	hr = this->pGrabber->GetCurrentBuffer(&cbBuffer, NULL);
	if(FAILED(hr))
	{
		return E_FAIL;
	}

	this->pBuffer = (BYTE*)CoTaskMemAlloc(cbBuffer);
    if (!this->pBuffer) 
    {
        return E_OUTOFMEMORY;
	}
	hr = this->pGrabber->GetCurrentBuffer(&cbBuffer, (long*)this->pBuffer);
	if(FAILED(hr))
	{
		return E_FAIL;
	}

	if ((mt.formattype == FORMAT_VideoInfo) && 
        (mt.cbFormat >= sizeof(VIDEOINFOHEADER)) &&
        (mt.pbFormat != NULL)) 
    {
        VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER*)mt.pbFormat;
		
		PreviewCapture::WriteBitmap(pFileName,&pVih->bmiHeader,mt.cbFormat - SIZE_PREHEADER, this->pBuffer, cbBuffer);
    }
    else 
    {
        // Invalid format.
        hr = VFW_E_INVALIDMEDIATYPE; 
    }
	return hr;
}

HRESULT PreviewCapture::WriteBitmap(PCWSTR pszFileName, BITMAPINFOHEADER *pBMI, size_t cbBMI, BYTE *pData, size_t cbData)
{
	HANDLE hFile = CreateFile(pszFileName, GENERIC_WRITE, 0, NULL, 
        CREATE_ALWAYS, 0, NULL);
    if (hFile == NULL)
    {
        return HRESULT_FROM_WIN32(GetLastError());
    }

	//pBMI->biCompression = BI_JPEG;

    BITMAPFILEHEADER bmf = { };

    bmf.bfType = 'MB';
    bmf.bfSize = cbBMI+ cbData + sizeof(bmf); 
    bmf.bfOffBits = sizeof(bmf) + cbBMI; 

    DWORD cbWritten = 0;
    BOOL result = WriteFile(hFile, &bmf, sizeof(bmf), &cbWritten, NULL);
    if (result)
    {
        result = WriteFile(hFile, pBMI, cbBMI, &cbWritten, NULL);
    }
    if (result)
    {
        result = WriteFile(hFile, pData, cbData, &cbWritten, NULL);
    }

    HRESULT hr = result ? S_OK : HRESULT_FROM_WIN32(GetLastError());

    CloseHandle(hFile);

    return hr;
}

HRESULT PreviewCapture::FindIntegratedWebCam()
{

	IMoniker *pLocalMonk = NULL;
	HRESULT hr = E_FAIL;
	while ((this->pEnum)->Next(1, &pLocalMonk, NULL) == S_OK)
    {
        IPropertyBag *pPropBag;
        hr = pLocalMonk->BindToStorage(0, 0, IID_PPV_ARGS(&pPropBag));
        if (FAILED(hr))
        {
            pLocalMonk->Release();
            continue;  
		}
        VARIANT var;
        VariantInit(&var);
            hr = pPropBag->Read(L"FriendlyName", &var, 0);
			if(FAILED(hr))
			{
				return E_INVALIDARG;
			}
			else
			{
				this->pMoniker = pLocalMonk;
				return hr;
			}
			/*
			int compareRes = strcmp("Integrated Webcam",(char*)var.bstrVal);
			if(compareRes == 1)
			{
				//break;
				this->pMoniker = pLocalMonk;
				return S_OK;
			}//if
			*/
		//}//if
	}//while
	return hr;
}

HRESULT PreviewCapture::AddCapturFilter()
{
	IBaseFilter *pLocalCap = NULL;
	if(this->pMoniker == NULL)
	{
		return S_FALSE;
	}
	HRESULT hr = this->pMoniker->BindToObject(0,0,IID_IBaseFilter,(void**)&pLocalCap);
	if(FAILED(hr))
	{
		return hr;
	}
	this->pCap = pLocalCap;
	hr = this->pGraph->AddFilter(this->pCap, L"Capture Filter");
	
	return hr;
}

HRESULT PreviewCapture::RestartPreview()
{
	if(this->pBuild == NULL || this->pGraph == NULL || this->pCap == NULL || this->_mediaControl == NULL || this->_vidWin == NULL)
	{
		return E_INVALIDARG;
	}
	HRESULT hr = this->_mediaControl->Run();
	if(FAILED(hr))
	{
		return hr;
	}
	hr = this->pCap->Run(INFINITE);
	if(FAILED(hr))
	{
		return hr;
	}
	return hr;
}

HRESULT PreviewCapture::RenderVedioOnWindow(HWND wHandle, int wWidth, int x, int y, int wHeight, bool wFullScreen)
{
	IMediaControl *pLocalMediaControl = NULL;
	IVideoWindow *pLocalVideoWindow = NULL;
	IMediaEventEx *pLocalEvent = NULL;
	IBaseFilter *pLocalRotateFilter = NULL;
	IBaseFilter *pLocalVMR = NULL;
	IPin *pLocalIQSOPPin = NULL;
	
	if(this->pBuild == NULL || this->pGraph == NULL || this->pCap == NULL || this->pGrabberBase == NULL)
	{
		return E_INVALIDARG;
	}
	
	HRESULT  hr = this->pGraph->QueryInterface(IID_IMediaControl,(void **)&pLocalMediaControl);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->_mediaControl = pLocalMediaControl;
	hr = this->pGraph->QueryInterface(IID_IVideoWindow, (void **)&pLocalVideoWindow);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->_vidWin = pLocalVideoWindow;
	hr = this->pGraph->QueryInterface(IID_IMediaEventEx, (void**)&pLocalEvent);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pEvent = pLocalEvent;
	hr = CoCreateInstance(CLSID_RotateVideoFilter,NULL,CLSCTX_INPROC_SERVER,IID_PPV_ARGS(&pLocalRotateFilter));
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pRotateVideFilter = pLocalRotateFilter;
	hr = this->pRotateVideFilter->QueryInterface(IID_IRotateVideoInterface,(void**)(&this->pRotateVideoInterface));
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pRotateVideoInterface->SetRotationAngle(0);
	
	hr = CoCreateInstance(CLSID_VideoMixingRenderer9,NULL,CLSCTX_INPROC_SERVER,IID_PPV_ARGS(&pLocalVMR));
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pVMR = pLocalVMR;
	/*
	hr = this->pBuild->RenderStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,this->pCap,this->pGrabberBase,NULL);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	hr = this->pGraph->AddFilter(this->pRotateVideFilter, L"PhotoApp Rotate Video Filter");
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	/*
	hr = this->pGraph->AddFilter(this->pVMR,L"Video Mixing Renderer 9");
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	hr = this->pBuild->RenderStream(&PIN_CATEGORY_CAPTURE,&MEDIATYPE_Video,this->pCap,NULL,this->pGrabberBase);
	if(FAILED(hr))
	{
		return E_FAIL;
	}	
	hr = this->pBuild->RenderStream(NULL,NULL,this->pGrabberBase,this->pRotateVideFilter,NULL);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	/*
	hr = this->pVMR->Run(INFINITE);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	*/
	hr = this->_mediaControl->Run();
	if(FAILED(hr))
	{
		return hr;
	}
	hr = this->pCap->Run(INFINITE);
	if(FAILED(hr))
	{
		return hr;
	}
	hr = this->pRotateVideFilter->FindPin(L"Out",&pLocalIQSOPPin);
	if(FAILED(hr))
	{
		return E_FAIL;
	}
	this->pIQSOutputPin = pLocalIQSOPPin;
	this->_vidWin->put_Owner((OAHWND)wHandle);
	this->_vidWin->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS);
	this->_vidWin->SetWindowPosition(x,y,wWidth,wHeight);
	return hr;
}

HRESULT PreviewCapture::SetPreviewWindow(HWND wHandle, int wWidth, int x, int y, int wHeight)
{
	if(this->_vidWin == NULL)
		return E_FAIL;
	HRESULT hr = this->_vidWin->SetWindowPosition(x,y,wWidth,wHeight);
	return hr;
}

HRESULT PreviewCapture::Stop()
{
	HRESULT hr = E_FAIL;
	if(this->pCap != NULL && this->_mediaControl != NULL)
	{
		hr = this->_mediaControl->Stop();
		if(FAILED(hr))
		{
			return hr;
		}
		hr = this->pCap->Stop();
	}
	return hr;
}

HRESULT PreviewCapture::Pause()
{
	HRESULT hr = E_FAIL;
	if(this->pCap != NULL && this->_mediaControl != NULL)
	{
		hr = this->_mediaControl->Pause();
		if(FAILED(hr))
		{
			return hr;
		}
		this->pCap->Pause();
	}
	return hr;
}

/*
ROTATE_NONE						= 0,
ROTATE_90_DEGREES_CLOCKWISE		= 1, //issue
ROTATE_180_DEGREES_CLOCKWISE	= 2,
ROTATE_270_DEGREES_CLOCKWISE	= 3, //issue
ROTATE_FLIP_VERTICAL			= 4,
ROTATE_FLIP_HORIZONTAL			= 5,
ROTATE_FLIP_DIAGONALLY			= 6 //issue
*/
HRESULT PreviewCapture::RotateVideo(int rAngle)
{
		HRESULT hr = E_FAIL;
		if(this->pRotateVideoInterface != NULL)
		{
			hr = this->pRotateVideoInterface->SetRotationAngle(rAngle);
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			hr = this->_mediaControl->Stop();
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			hr = this->pCap->Stop();
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			if(this->pIQSOutputPin == NULL)
			{
				return E_FAIL;
			}
			hr = this->pGraph->Reconnect(this->pIQSOutputPin);
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			hr = this->_mediaControl->Run();
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			hr = this->pCap->Run(INFINITE);
			if(FAILED(hr))
			{
				return E_FAIL;
			}
			return S_OK;
		}
		else
		{
			return hr;
		}
}

HRESULT PreviewCapture::ReleaseResources()
{
	HRESULT hr = E_FAIL;
	
	if(this->pGraph != NULL)
		hr = this->pGraph->Release();
	if(this->pBuild != NULL)
		hr = this->pBuild->Release();
	if(this->pCap != NULL)
		hr = this->pCap->Release();
	if(this->pGrabberBase != NULL)
		hr = this->pGrabberBase->Release();
	if(this->_vidWin != NULL)
		hr = this->_vidWin->Release();
	if(this->_mediaControl != NULL)
		hr = this->_mediaControl->Release();
	if(this->_videoControl != NULL)
		hr = this->_videoControl->Release();
	if(this->pEnum != NULL)
		hr = this->pEnum->Release();
	if(this->pMoniker != NULL)
		hr = this->pMoniker->Release();
	if(this->pGrabber != NULL)
		hr = this->pGrabber->Release();
	if(this->pEvent != NULL)
		hr = this->pEvent->Release();

	if(this->pRotateVideFilter != NULL)
		hr = this->pRotateVideFilter->Release();

	if(this->pRotateVideoInterface != NULL)
		hr = this->pRotateVideoInterface->Release();

	if(this->pVMR != NULL)
		hr = this->pVMR->Release();

		CoUninitialize();
	return hr;
}
